use std::sync::Arc;

use axum::{
    Json,
    extract::{Json as ExtractJson, State},
    response::IntoResponse,
    http::StatusCode
};
use serde::{Deserialize, Serialize};
use bb8::Pool;
use bb8_postgres::PostgresConnectionManager;
use tokio_postgres::NoTls;

use crate::launcher::auth::{generate_access_token,verify_hash};


//
// Структуры запросов/ответов
//
#[derive(Deserialize)]
pub struct AuthRequest {
    username: String,
    password: String,
}

#[derive(Serialize)]
struct AuthResponse {
    pub username: String,
    pub uuid: String,
    pub access_token: String,
}


//
// Авторизация по паролю
//
pub async fn from_password(
    State(pool): State<Arc<Pool<PostgresConnectionManager<NoTls>>>>,
    ExtractJson(payload): ExtractJson<AuthRequest>,
) -> impl IntoResponse {
    let conn = match pool.get().await {
        Ok(conn) => conn, Err(_e) => return (StatusCode::INTERNAL_SERVER_ERROR, "Failed to get DB connection").into_response()
    };

    let query = "SELECT password, uuid FROM accounts WHERE username = $1";
    let row = match conn.query_one(query, &[&payload.username]).await {
        Ok(row) => row,
        Err(_) => return (StatusCode::UNAUTHORIZED, "Invalid username or password").into_response()
    };

    let db_uuid = row.get("uuid");
    let stored_password: String = row.get("password");
    
    if !verify_hash(&stored_password, &payload.password) {
        return (StatusCode::UNAUTHORIZED, "Invalid username or password").into_response();
    }

    let access_token = generate_access_token();
    let update_query = "UPDATE accounts SET access_token = $1 WHERE username = $2";
    if conn.execute(update_query, &[&access_token, &payload.username]).await.is_err() {
        return (StatusCode::INTERNAL_SERVER_ERROR, "Failed to update access token").into_response();
    }

    let response = AuthResponse {
        username: payload.username,
        uuid: db_uuid,
        access_token,
    };

    (StatusCode::OK, Json(response)).into_response()
}
