use std::sync::Arc;

use axum::{
    Json,
    extract::{Json as ExtractJson, State},
    response::IntoResponse,
    http::StatusCode
};
use serde::{Deserialize, Serialize};
use bb8::Pool;
use bb8_postgres::PostgresConnectionManager;
use tokio_postgres::NoTls;

use crate::launcher::auth::generate_access_token;

//
// Структуры запросов/ответов
//
#[derive(Deserialize)]
pub struct AuthRequestToken {
    uuid: String,
    access_token: String,
}

#[derive(Serialize)]
struct AuthResponse {
    pub username: String,
    pub uuid: String,
    pub access_token: String,
}


//
// Авторизация по токену
//
pub async fn from_token (
    State(pool): State<Arc<Pool<PostgresConnectionManager<NoTls>>>>,
    ExtractJson(payload): ExtractJson<AuthRequestToken>,
) -> impl IntoResponse {
    let conn = match pool.get().await {
        Ok(conn) => conn,
        Err(_) => return (StatusCode::INTERNAL_SERVER_ERROR, "Failed to get DB connection").into_response()
    };
    
    // Обновление токена и вставка
    let access_token = generate_access_token();
    let query = "UPDATE accounts SET access_token = $1 WHERE uuid = $2 AND access_token = $3 RETURNING username, access_token";
    let row = match conn.query_one(query, &[&access_token, &payload.uuid, &payload.access_token]).await {
        Ok(row) => row,
        Err(_) => return (StatusCode::UNAUTHORIZED, "Invalid UUID or access token").into_response(),
    };

    let username: String = row.get("username");

    // Ответ
    let response = AuthResponse {
        username,
        uuid: payload.uuid,
        access_token,
    };
    (StatusCode::OK, Json(response)).into_response()
}
